@@ -28,7 +28,10 @@ else:
 
 
 __all__ = ["tabulate", "tabulate_formats", "simple_separated_format"]
-__version__ = "0.7.2"
+__version__ = "0.7.3"
+
+
+MIN_PADDING = 2
 
 
 Line = namedtuple("Line", ["begin", "hline", "sep", "end"])
@@ -108,10 +111,24 @@ def _mediawiki_row_with_attrs(separator, cell_values, colwidths, colaligns):
     return (separator + colsep.join(values_with_attrs)).rstrip()
 
 
-def _latex_line_begin_tabular(colwidths, colaligns):
+def _latex_line_begin_tabular(colwidths, colaligns, booktabs=False):
     alignment = { "left": "l", "right": "r", "center": "c", "decimal": "r" }
     tabular_columns_fmt = "".join([alignment.get(a, "l") for a in colaligns])
-    return "\\begin{tabular}{" + tabular_columns_fmt + "}\n\hline"
+    return "\n".join(["\\begin{tabular}{" + tabular_columns_fmt + "}",
+                      "\\toprule" if booktabs else "\hline"])
+
+LATEX_ESCAPE_RULES = {r"&": r"\&", r"%": r"\%", r"$": r"\$", r"#": r"\#",
+                      r"_": r"\_", r"^": r"\^{}", r"{": r"\{", r"}": r"\}",
+                      r"~": r"\textasciitilde{}", "\\": r"\textbackslash{}",
+                      r"<": r"\ensuremath{<}", r">": r"\ensuremath{>}"}
+
+
+def _latex_row(cell_values, colwidths, colaligns):
+    def escape_char(c):
+        return LATEX_ESCAPE_RULES.get(c, c)
+    escaped_values = ["".join(map(escape_char, cell)) for cell in cell_values]
+    rowfmt = DataRow("", "&", "\\\\")
+    return _build_simple_row(escaped_values, rowfmt)
 
 
 _table_formats = {"simple":
@@ -176,8 +193,16 @@ _table_formats = {"simple":
                               linebelowheader=Line("\\hline", "", "", ""),
                               linebetweenrows=None,
                               linebelow=Line("\\hline\n\\end{tabular}", "", "", ""),
-                              headerrow=DataRow("", "&", "\\\\"),
-                              datarow=DataRow("", "&", "\\\\"),
+                              headerrow=_latex_row,
+                              datarow=_latex_row,
+                              padding=1, with_header_hide=None),
+                  "latex_booktabs":
+                  TableFormat(lineabove=partial(_latex_line_begin_tabular, booktabs=True),
+                              linebelowheader=Line("\\midrule", "", "", ""),
+                              linebetweenrows=None,
+                              linebelow=Line("\\bottomrule\n\\end{tabular}", "", "", ""),
+                              headerrow=_latex_row,
+                              datarow=_latex_row,
                               padding=1, with_header_hide=None),
                   "tsv":
                   TableFormat(lineabove=None, linebelowheader=None,
@@ -190,8 +215,8 @@ _table_formats = {"simple":
 tabulate_formats = list(sorted(_table_formats.keys()))
 
 
-_invisible_codes = re.compile("\x1b\[\d*m")  # ANSI color codes
-_invisible_codes_bytes = re.compile(b"\x1b\[\d*m")  # ANSI color codes
+_invisible_codes = re.compile(r"\x1b\[\d*m|\x1b\[\d*\;\d*\;\d*m")  # ANSI color codes
+_invisible_codes_bytes = re.compile(b"\x1b\[\d*m|\x1b\[\d*\;\d*\;\d*m")  # ANSI color codes
 
 
 def simple_separated_format(separator):
@@ -212,7 +237,7 @@ def _isconvertible(conv, string):
     try:
         n = conv(string)
         return True
-    except ValueError:
+    except (ValueError, TypeError):
         return False
 
 
@@ -445,7 +470,10 @@ def _format(val, valtype, floatfmt, missingval=""):
     if valtype in [int, _text_type]:
         return "{0}".format(val)
     elif valtype is _binary_type:
-        return _text_type(val, "ascii")
+        try:
+            return _text_type(val, "ascii")
+        except TypeError:
+            return _text_type(val)
     elif valtype is float:
         return format(float(val), floatfmt)
     else:
@@ -472,6 +500,10 @@ def _normalize_tabular_data(tabular_data, headers):
 
     * list of named tuples (usually used with headers="keys")
 
+    * list of dicts (usually used with headers="keys")
+
+    * list of OrderedDicts (usually used with headers="keys")
+
     * 2D NumPy arrays
 
     * NumPy record arrays (usually used with headers="keys")
@@ -514,9 +546,42 @@ def _normalize_tabular_data(tabular_data, headers):
         elif (headers == "keys"
               and len(rows) > 0
               and isinstance(rows[0], tuple)
-              and hasattr(rows[0], "_fields")): # namedtuple
+              and hasattr(rows[0], "_fields")):
+            # namedtuple
             headers = list(map(_text_type, rows[0]._fields))
-        elif headers == "keys" and len(rows) > 0:  # keys are column indices
+        elif (len(rows) > 0
+              and isinstance(rows[0], dict)):
+            # dict or OrderedDict
+            uniq_keys = set() # implements hashed lookup
+            keys = [] # storage for set
+            if headers == "firstrow":
+                firstdict = rows[0] if len(rows) > 0 else {}
+                keys.extend(firstdict.keys())
+                uniq_keys.update(keys)
+                rows = rows[1:]
+            for row in rows:
+                for k in row.keys():
+                    #Save unique items in input order
+                    if k not in uniq_keys:
+                        keys.append(k)
+                        uniq_keys.add(k)
+            if headers == 'keys':
+                headers = keys
+            elif isinstance(headers, dict):
+                # a dict of headers for a list of dicts
+                headers = [headers.get(k, k) for k in keys]
+                headers = list(map(_text_type, headers))
+            elif headers == "firstrow":
+                if len(rows) > 0:
+                    headers = [firstdict.get(k, k) for k in keys]
+                    headers = list(map(_text_type, headers))
+                else:
+                    headers = []
+            elif headers:
+                raise ValueError('headers for a list of dicts is not a dict or a keyword')
+            rows = [[row.get(k) for k in keys] for row in rows]
+        elif headers == "keys" and len(rows) > 0:
+            # keys are column indices
             headers = list(map(_text_type, range(len(rows[0]))))
 
     # take headers from the first row if necessary
@@ -524,7 +589,7 @@ def _normalize_tabular_data(tabular_data, headers):
         headers = list(map(_text_type, rows[0])) # headers should be strings
         rows = rows[1:]
 
-    headers = list(headers)
+    headers = list(map(_text_type,headers))
     rows = list(map(list,rows))
 
     # pad with empty headers for initial columns if necessary
@@ -551,8 +616,9 @@ def tabulate(tabular_data, headers=[], tablefmt="simple",
 
     The first required argument (`tabular_data`) can be a
     list-of-lists (or another iterable of iterables), a list of named
-    tuples, a dictionary of iterables, a two-dimensional NumPy array,
-    NumPy record array, or a Pandas' dataframe.
+    tuples, a dictionary of iterables, an iterable of dictionaries,
+    a two-dimensional NumPy array, NumPy record array, or a Pandas'
+    dataframe.
 
 
     Table headers
@@ -608,7 +674,7 @@ def tabulate(tabular_data, headers=[], tablefmt="simple",
 
     Various plain-text table formats (`tablefmt`) are supported:
     'plain', 'simple', 'grid', 'pipe', 'orgtbl', 'rst', 'mediawiki',
-    and 'latex'. Variable `tabulate_formats` contains the list of
+     'latex', and 'latex_booktabs'. Variable `tabulate_formats` contains the list of
     currently supported formats.
 
     "plain" format doesn't use any pseudographics to draw tables,
@@ -734,8 +800,19 @@ def tabulate(tabular_data, headers=[], tablefmt="simple",
     \\hline
     \\end{tabular}
 
-    """
+    "latex_booktabs" produces a tabular environment of LaTeX document markup
+    using the booktabs.sty package:
 
+    >>> print(tabulate([["spam", 41.9999], ["eggs", "451.0"]], tablefmt="latex_booktabs"))
+    \\begin{tabular}{lr}
+    \\toprule
+     spam &  41.9999 \\\\
+     eggs & 451      \\\\
+    \\bottomrule
+    \end{tabular}
+    """
+    if tabular_data is None:
+        tabular_data = []
     list_of_lists, headers = _normalize_tabular_data(tabular_data, headers)
 
     # optimization: look for ANSI control codes once,
@@ -756,15 +833,17 @@ def tabulate(tabular_data, headers=[], tablefmt="simple",
 
     # align columns
     aligns = [numalign if ct in [int,float] else stralign for ct in coltypes]
-    minwidths = [width_fn(h)+2 for h in headers] if headers else [0]*len(cols)
+    minwidths = [width_fn(h) + MIN_PADDING for h in headers] if headers else [0]*len(cols)
     cols = [_align_column(c, a, minw, has_invisible)
             for c, a, minw in zip(cols, aligns, minwidths)]
 
     if headers:
         # align headers and add headers
-        minwidths = [max(minw, width_fn(c[0])) for minw, c in zip(minwidths, cols)]
+        t_cols = cols or [['']] * len(headers)
+        t_aligns = aligns or [stralign] * len(headers)
+        minwidths = [max(minw, width_fn(c[0])) for minw, c in zip(minwidths, t_cols)]
         headers = [_align_header(h, a, minw)
-                   for h, a, minw in zip(headers, aligns, minwidths)]
+                   for h, a, minw in zip(headers, t_aligns, minwidths)]
         rows = list(zip(*cols))
     else:
         minwidths = [width_fn(c[0]) for c in cols]
