@@ -5,6 +5,7 @@
 from __future__ import print_function
 from __future__ import unicode_literals
 from tabulate import tabulate
+from common import assert_equal, assert_in, assert_raises
 
 
 def test_iterable_of_iterables():
@@ -16,9 +17,7 @@ def test_iterable_of_iterables():
          '5  4  3  2  1',
          '-  -  -  -  -'])
     result   = tabulate(ii)
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
 
 
 def test_iterable_of_iterables_headers():
@@ -30,9 +29,7 @@ def test_iterable_of_iterables_headers():
          '  0    1    2    3    4',
          '  5    4    3    2    1'])
     result   = tabulate(ii, "abcde")
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
 
 
 def test_iterable_of_iterables_firstrow():
@@ -44,9 +41,7 @@ def test_iterable_of_iterables_firstrow():
          '  0    1    2    3    4',
          '  5    4    3    2    1'])
     result   = tabulate(ii, "firstrow")
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
 
 
 def test_list_of_lists():
@@ -58,9 +53,7 @@ def test_list_of_lists():
         'a   one              1',
         'b   two'])
     result   = tabulate(ll, headers=["string","number"])
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
 
 
 def test_list_of_lists_firstrow():
@@ -72,9 +65,7 @@ def test_list_of_lists_firstrow():
         'a   one              1',
         'b   two'])
     result   = tabulate(ll, headers="firstrow")
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
 
 
 def test_list_of_lists_keys():
@@ -86,9 +77,7 @@ def test_list_of_lists_keys():
         'a    one    1',
         'b    two'])
     result   = tabulate(ll, headers="keys")
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
 
 
 def test_dict_like():
@@ -113,10 +102,7 @@ def test_dict_like():
         '104'])
     result    = tabulate(dd, "keys")
     print("Keys' order: %s" % dd.keys())
-    print("Expected 1:\n%s\n" % expected1)
-    print("Expected 2:\n%s\n" % expected2)
-    print("Got:\n%s\n" % result)
-    assert result in [expected1, expected2]
+    assert_in(result, [expected1, expected2])
 
 
 def test_numpy_2d():
@@ -131,9 +117,7 @@ def test_numpy_2d():
             ' 32     62.5  108',
             '171.5  256    364.5'])
         result   = tabulate(na, ["a", "b", "c"])
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_numpy_2d is skipped")
         pass   # this test is optional
@@ -150,9 +134,7 @@ def test_numpy_2d_firstrow():
             ' 64  125   216',
             '343  512   729'])
         result   = tabulate(na, headers="firstrow")
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%r\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_numpy_2d_firstrow is skipped")
         pass   # this test is optional
@@ -170,9 +152,7 @@ def test_numpy_2d_keys():
             ' 32     62.5  108',
             '171.5  256    364.5'])
         result   = tabulate(na, headers="keys")
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_numpy_2d_keys is skipped")
         pass   # this test is optional
@@ -192,9 +172,7 @@ def test_numpy_record_array():
             "Bob    27  175",
             "-----  --  -----" ])
         result   = tabulate(na)
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_numpy_2d_keys is skipped")
         pass   # this test is optional
@@ -214,9 +192,7 @@ def test_numpy_record_array_keys():
             "Alice      23     169.5",
             "Bob        27     175"  ])
         result   = tabulate(na, headers="keys")
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_numpy_2d_keys is skipped")
         pass   # this test is optional
@@ -236,9 +212,7 @@ def test_numpy_record_array_headers():
             "Alice          23  169.5",
             "Bob            27  175" ])
         result   = tabulate(na, headers=["person", "years", "cm"])
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_numpy_2d_keys is skipped")
         pass   # this test is optional
@@ -255,9 +229,7 @@ def test_pandas():
             'a   one              1',
             'b   two            nan'])
         result   = tabulate(df, headers=["string", "number"])
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_pandas is skipped")
         pass   # this test is optional
@@ -275,9 +247,7 @@ def test_pandas_firstrow():
             '---  -----  -----',
             'b    two      nan'])
         result   = tabulate(df, headers="firstrow")
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_pandas_firstrow is skipped")
         pass   # this test is optional
@@ -296,9 +266,7 @@ def test_pandas_keys():
              'a   one              1',
              'b   two            nan'])
         result   = tabulate(df, headers="keys")
-        print("Expected:\n%s\n" % expected)
-        print("Got:\n%s\n" % result)
-        assert expected == result
+        assert_equal(expected, result)
     except ImportError:
         print("test_pandas_keys is skipped")
         pass   # this test is optional
@@ -315,9 +283,7 @@ def test_list_of_namedtuples():
         '3  4',
         '-  -'])
     result = tabulate(lt)
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
 
 
 def test_list_of_namedtuples_keys():
@@ -331,6 +297,105 @@ def test_list_of_namedtuples_keys():
         '    1      2',
         '    3      4'])
     result = tabulate(lt, headers="keys")
-    print("Expected:\n%s\n" % expected)
-    print("Got:\n%s\n" % result)
-    assert expected == result
+    assert_equal(expected, result)
+
+
+def test_list_of_dicts():
+    "Input: a list of dictionaries."
+    lod = [{'foo' : 1, 'bar' : 2}, {'foo' : 3, 'bar' : 4}]
+    expected1 = "\n".join([
+        '-  -',
+        '1  2',
+        '3  4',
+        '-  -'])
+    expected2 = "\n".join([
+        '-  -',
+        '2  1',
+        '4  3',
+        '-  -'])
+    result = tabulate(lod)
+    assert_in(result, [expected1, expected2])
+
+
+def test_list_of_dicts_keys():
+    "Input: a list of dictionaries, with keys as headers."
+    lod = [{'foo' : 1, 'bar' : 2}, {'foo' : 3, 'bar' : 4}]
+    expected1 = "\n".join([
+        '  foo    bar',
+        '-----  -----',
+        '    1      2',
+        '    3      4'])
+    expected2 = "\n".join([
+        '  bar    foo',
+        '-----  -----',
+        '    2      1',
+        '    4      3'])
+    result = tabulate(lod, headers="keys")
+    assert_in(result, [expected1, expected2])
+
+
+def test_list_of_dicts_with_missing_keys():
+    "Input: a list of dictionaries, with missing keys."
+    lod = [{"foo": 1}, {"bar": 2}, {"foo":4, "baz": 3}]
+    expected = "\n".join([
+        '  foo    bar    baz',
+        '-----  -----  -----',
+        '    1',
+        '           2',
+        '    4             3'])
+    result = tabulate(lod, headers="keys")
+    assert_equal(expected, result)
+
+
+def test_list_of_dicts_firstrow():
+    "Input: a list of dictionaries, with the first dict as headers."
+    lod = [{'foo' : "FOO", 'bar' : "BAR"}, {'foo' : 3, 'bar': 4, 'baz': 5}]
+    # if some key is missing in the first dict, use the key name instead
+    expected1 = "\n".join([
+        '  FOO    BAR    baz',
+        '-----  -----  -----',
+        '    3      4      5'])
+    expected2 = "\n".join([
+        '  BAR    FOO    baz',
+        '-----  -----  -----',
+        '    4      3      5'])
+    result = tabulate(lod, headers="firstrow")
+    assert_in(result, [expected1, expected2])
+
+
+def test_list_of_dicts_with_dict_of_headers():
+    "Input: a dict of user headers for a list of dicts (issue #23)"
+    table = [{"letters": "ABCDE", "digits": 12345}]
+    headers = {"digits": "DIGITS", "letters": "LETTERS"}
+    expected1 = "\n".join([
+        '  DIGITS  LETTERS',
+        '--------  ---------',
+        '   12345  ABCDE'])
+    expected2 = "\n".join([
+        'LETTERS      DIGITS',
+        '---------  --------',
+        'ABCDE         12345'])
+    result = tabulate(table, headers=headers)
+    assert_in(result, [expected1, expected2])
+
+
+def test_list_of_dicts_with_list_of_headers():
+    "Input: a list of headers for a list of dicts, raise ValueError (issue #23)"
+    table = [{"letters": "ABCDE", "digits": 12345}]
+    headers = ["DIGITS", "LETTERS"]
+    with assert_raises(ValueError):
+        tabulate(table, headers=headers)
+
+
+def test_py27orlater_list_of_ordereddicts():
+    "Input: a list of OrderedDicts."
+    from collections import OrderedDict
+    od = OrderedDict([('b', 1), ('a', 2)])
+    lod = [od, od]
+    expected = "\n".join([
+        '  b    a',
+        '---  ---',
+        '  1    2',
+        '  1    2'])
+    result = tabulate(lod, headers="keys")
+    assert_equal(expected, result)
