@@ -106,3 +106,66 @@ def test_column_type_of_bytestring_columns():
     result = _column_type([b"foo", b"bar"])
     expected = _binary_type
     assert_equal(result, expected)
+
+
+def test_numeric_column_headers():
+    "Regression: numbers as column headers (issue #22)"
+    result = tabulate([[1],[2]], [42])
+    expected = '  42\n----\n   1\n   2'
+    assert_equal(result, expected)
+
+    lod = [dict((p,i) for p in range(5)) for i in range(5)]
+    result = tabulate(lod, "keys")
+    expected = "\n".join([
+        "  0    1    2    3    4",
+        "---  ---  ---  ---  ---",
+        "  0    0    0    0    0",
+        "  1    1    1    1    1",
+        "  2    2    2    2    2",
+        "  3    3    3    3    3",
+        "  4    4    4    4    4",])
+    assert_equal(result, expected)
+
+
+def test_88_256_ANSI_color_codes():
+    "Regression: color codes for terminals with 88/256 colors (issue #26)"
+    colortable = [('\x1b[48;5;196mred\x1b[49m',
+                   '\x1b[38;5;196mred\x1b[39m')]
+    colorlessheaders = ('background', 'foreground')
+    formatted = tabulate(colortable, colorlessheaders, 'pipe')
+    expected = "\n".join([
+        '| background   | foreground   |',
+        '|:-------------|:-------------|',
+        '| \x1b[48;5;196mred\x1b[49m          | \x1b[38;5;196mred\x1b[39m          |'])
+    print("expected: %r\n\ngot:      %r\n" % (expected, formatted))
+    assert_equal(expected, formatted)
+
+
+def test_column_with_mixed_value_types():
+    "Regression: mixed value types in the same column (issue #31)"
+    expected = '\n'.join([
+        '-----',
+        '',
+        'a',
+        'я',
+        '0',
+        'False',
+        '-----',
+    ])
+    data = [[None], ['a'], ['\u044f'], [0], [False]]
+    table = tabulate(data)
+    assert_equal(table, expected)
+
+
+def test_latex_escape_special_chars():
+    "Regression: escape special characters in LaTeX output (issue #32)"
+    expected = "\n".join([
+        r'\begin{tabular}{l}',
+        r'\hline',
+        r' foo\^{}bar     \\',
+        r'\hline',
+        r' \&\%\^{}\_\$\#\{\}\ensuremath{<}\ensuremath{>}\textasciitilde{} \\',
+        r'\hline',
+        r'\end{tabular}'])
+    result = tabulate([["&%^_$#{}<>~"]], ["foo^bar"], tablefmt="latex")
+    assert_equal(result, expected)
